import pytest
from pathlib import Path
from rq_controller.common import PyProjection, PyProjectionGeometry, PyRegionOfIntrest, PyVolume
from rq_controller.common.io.writer import BaseDataWriter

@pytest.fixture
def base_data_writer():
    return BaseDataWriter(
        porjection_geometry_suffix=".geom",
        projection_suffix=".proj",
        region_of_intrest_suffix=".roi",
        volume_suffix=".vol"
    )

def test_initialization(base_data_writer):
    assert base_data_writer.porjection_geometry_suffix == ".geom"
    assert base_data_writer.projection_suffix == ".proj"
    assert base_data_writer.region_of_intrest_suffix == ".roi"
    assert base_data_writer.volume_suffix == ".vol"

def test_write_projection_geometry_not_implemented(base_data_writer):
    with pytest.raises(NotImplementedError):
        base_data_writer.write_projection_geometry(Path("/fake/path"), PyProjectionGeometry.dummy())

def test_write_projection_not_implemented(base_data_writer):
    with pytest.raises(NotImplementedError):
        base_data_writer.write_projection(Path("/fake/path"), PyProjection.dummy())

def test_write_region_of_intrest_not_implemented(base_data_writer):
    with pytest.raises(NotImplementedError):
        base_data_writer.write_region_of_intrest(Path("/fake/path"), PyRegionOfIntrest.dummy())

def test_write_volume_not_implemented(base_data_writer):
    with pytest.raises(NotImplementedError):
        base_data_writer.write_volume(Path("/fake/path"), PyVolume.dummy())

def test_get_next_projection_save_path(base_data_writer, tmp_path):
    (tmp_path / "projection_00001.proj").touch()
    expected_path = tmp_path / "projection_00002.proj"
    assert base_data_writer.get_next_projection_save_path(tmp_path) == expected_path

def test_get_projection_save_path_i(base_data_writer, tmp_path):
    expected_path = tmp_path / "projection_00005.proj"
    assert base_data_writer.get_projection_save_path_i(tmp_path, 5) == expected_path

def test_get_next_projection_geometry_save_path(base_data_writer, tmp_path):
    (tmp_path / "geometry_00001.geom").touch()
    expected_path = tmp_path / "geometry_00002.geom"
    assert base_data_writer.get_next_projection_geometry_save_path(tmp_path) == expected_path

def test_get_projection_geometry_save_path_i(base_data_writer, tmp_path):
    expected_path = tmp_path / "geometry_00005.geom"
    assert base_data_writer.get_projection_geometry_save_path_i(tmp_path, 5) == expected_path

def test_get_next_region_of_intrest_save_path(base_data_writer, tmp_path):
    (tmp_path / "roi_00001.roi").touch()
    expected_path = tmp_path / "roi_00002.roi"
    assert base_data_writer.get_next_region_of_intrest_save_path(tmp_path) == expected_path

def test_get_region_of_intrest_save_path_i(base_data_writer, tmp_path):
    expected_path = tmp_path / "roi_00005.roi"
    assert base_data_writer.get_region_of_intrest_save_path_i(tmp_path, 5) == expected_path

def test_get_next_volume_save_path(base_data_writer, tmp_path):
    (tmp_path / "volume_00001.vol").touch()
    expected_path = tmp_path / "volume_00002.vol"
    assert base_data_writer.get_next_volume_save_path(tmp_path) == expected_path

def test_get_volume_save_path_i(base_data_writer, tmp_path):
    expected_path = tmp_path / "volume_00005.vol"
    assert base_data_writer.get_volume_save_path_i(tmp_path, 5) == expected_path

def test_number_of_projection_geometries(base_data_writer, tmp_path):
    (tmp_path / "geometry_00001.geom").touch()
    (tmp_path / "geometry_00002.geom").touch()
    assert base_data_writer.number_of_projection_geometries(tmp_path) == 2

def test_number_of_projections(base_data_writer, tmp_path):
    (tmp_path / "projection_00001.proj").touch()
    (tmp_path / "projection_00002.proj").touch()
    assert base_data_writer.number_of_projections(tmp_path) == 2

def test_number_of_region_of_intrests(base_data_writer, tmp_path):
    (tmp_path / "roi_00001.roi").touch()
    (tmp_path / "roi_00002.roi").touch()
    assert base_data_writer.number_of_region_of_intrests(tmp_path) == 2

def test_number_of_volumes(base_data_writer, tmp_path):
    (tmp_path / "volume_00001.vol").touch()
    (tmp_path / "volume_00002.vol").touch()
    assert base_data_writer.number_of_volumes(tmp_path) == 2